<?php
/*
Plugin Name: Latest MobileMe Photos
Plugin URI:
Description: Displays your recently uploaded or changed photos from your MobileMe webgallery.
Author: Kay Butter
Version: 0.1
Author URI: http://www.dertranszendente.de
*/

/*  Copyright 2009 Kay Butter (kay.butter@me.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

require_once( ABSPATH . WPINC . '/class-snoopy.php' );

define("MML_domain", "latest");
define("MML_widget", "MobileMeLatest");


add_action('activate_latest-mobileme-photos/latest.php', 'MML_install');
function MML_install() {
	$default = array(
		"username" => "",
		"count" => 12,
		"check_frequency" => 3600,
		"last_check" => 0,
		"last_update" => 0,
		"size" => 40
	);
	add_option("MML_settings", $default, '', yes);
	add_option("MML_cache", false, '', yes);
}

add_action('plugins_loaded', "MML_init");
function MML_init() {
	load_plugin_textdomain(MML_domain, PLUGINDIR."/".dirname(plugin_basename(__FILE__)), dirname(plugin_basename(__FILE__)));
	register_sidebar_widget(MML_widget, 'MML_widget');
	wp_register_widget_control(MML_widget, "tada", 'MML_widget_control');
}

function MML_widget_control() {
	$settings = get_option("MML_settings");
	$widget_data = $_POST[MML_widgetname];
	if($widget_data["submit"]) {
		$settings["username"] = $widget_data["username"];
		$settings["count"] = $widget_data["count"];
		$settings["check_frequency"] = $widget_data["check_frequency"];
		$settings["size"] = $widget_data["size"];
		update_option("MML_settings", $settings);
	}
	$check_frequency = $settings["check_frequency"];
	$count = $settings["count"];
	$username = $settings["username"];
	$size = $settings["size"];
	
	?>
	<p><label for="MML_username"><?php echo __("MobileMe Username", MML_domain); ?>:</label><input type="text" name="<?php echo MML_widgetname; ?>[username]" value="<?php echo $username; ?>" id="MML_username" /></p>
	<p><label for="MML_count"><?php echo __("Display Limit", MML_domain); ?>:</label><input type="text" name="<?php echo MML_widgetname; ?>[count]" value="<?php echo $count; ?>" id="MML_count" /></p>
	<p><label for="MML_size"><?php echo __("Thumbnail Size", MML_domain); ?>:</label><input type="text" name="<?php echo MML_widgetname; ?>[size]" value="<?php echo $size; ?>" id="MML_size" /></p>
	<p><label for="MML_check_frequency"><?php echo __("Update Frequency", MML_domain); ?>:</label><input type="text" name="<?php echo MML_widgetname; ?>[check_frequency]" value="<?php echo $check_frequency; ?>" id="MML_check_frequency" /></p>
	<input type="hidden" name="<?php echo MML_widgetname; ?>[submit]" value="1"/>
	<?php
	
}

function MML_widget($args) {
	extract($args, EXTR_SKIP);
	echo $before_widget;
	echo $before_title.__("Latest Photos", MML_domain).$after_title;
	mm_latest_photos();
	echo $after_widget;
}

function mm_get_latest_photos() {
	$currentTime = time();
	$settings = get_option("MML_settings");
	$cache = get_option("MML_cache");

	$force_update = $_REQUEST['force_update'];	
	$check_for_updates = $force_update || !(($currentTime <= $settings["last_check"]+$settings["check_frequency"]) && is_array($cache));


	if(!$check_for_updates) {
		$photos = $cache;
	} else {
		$gallery = new MMGallery($settings["username"]);
		if($gallery->needsUpdate($settings["last_update"]) || $force_update) {
			$photos = $gallery->getRecentPhotos();
			$photos = array_slice (  $photos , 0, $settings["count"]);
			$cache = $photos;
			$settings["last_update"] = $currentTime;
			update_option("MML_cache", $cache);
		} else {
			$photos = $cache;
		}
		$settings["last_check"] = $currentTime;
		update_option("MML_settings", $settings);
		
	}
	return $photos;
}

function mm_latest_photos() {
	try {
		$photos = mm_get_latest_photos();
		$settings = get_option("MML_settings");
		echo "<div class='MMPhotos'>";
		foreach ($photos as $key => $photo) {
			echo sprintf('<a href="%s"><img width="%s" height="%s" src="%s" alt="%s" title="%s" /></a>', $photo["link"], $settings["size"], $settings["size"], $photo["square"], attribute_Escape($photo["title"]), attribute_Escape($photo["title"]));
		}
		echo "</div>";
	} catch(Exception $e) {
		echo __("Error: Images could not be loaded.", MML_domain);
	}
}


/**
* MMGallery
*/
class MMGallery {
	
	private $username;
	private $categoryFeedPattern = "http://photocast.mac.com/--username--/rss";
	private $categoryFeed = null;
	
	function __construct($username) {
		$this->username = $username;
	}
	
	private function getFeed($url) {
		$snoopy = new Snoopy;
		$snoopy->fetch($url);
		if(!$snoopy->results) {
			throw new Exception("Feed konnte nicht geladen werden.");
		}
		try{
			$xml = @new SimpleXMLElement($snoopy->results);
		} catch(Exception $e) {
			throw new Exception("Feed konnte nicht geladen werden.");
		}
		return $xml;
	}
	private function getCategoryFeed() {
		if(!$this->categoryFeed) {
			$url = str_replace("--username--", $this->username, $this->categoryFeedPattern);
			$this->categoryFeed = $this->getFeed($url);
		}

		return $this->categoryFeed;
	}
	
	private function getCategoryUrls() {
		$feed = $this->getCategoryFeed();
		$list = array();
		foreach($feed->channel->item as $item) {
			$list[] = str_replace("http://gallery.mac.com", "http://photocast.mac.com", (String)$item->link)."/rss";
		}
		return $list;
	}
	private function itemToPhoto($item) {
		$photo = array();
		$photo["link"] = (String)$item->link;
		$photo["square"] = ((String)$item->link).".jpg?derivative=square&amp;source=web.jpg&amp;type=square";
		$photo["title"] = (String)$item->title;
		$photo["date"] = strtotime((String)$item->pubDate);
		return $photo;
	}
	private function getPhotosFromFeed($url) {
		$feed = $this->getFeed($url);
		$photos = array();
		foreach($feed->channel->item as $item) {
			$photos[] = $this->itemToPhoto($item);
		}
		return $photos;
	}
	public function needsUpdate($last_updated) {
		$feed = $this->getCategoryFeed();
		$pubdate = strtotime((String)$feed->channel->pubDate);
		return $last_updated < $pubdate;
	}
	public function getRecentPhotos() {
		$urls = $this->getCategoryUrls();
		$photos = array();
		foreach($urls as $url) {
			$new = $this->getPhotosFromFeed($url);
			$photos = array_merge($photos, $new);
		}
		usort($photos, array(&$this, "comparePhotos"));
		return $photos;
	}

	private function comparePhotos($photo1, $photo2) {
		if($photo1["date"] == $photo2["date"]) {
			return 0;
		}
		return ($photo1["date"] < $photo2["date"]) ? 1 : -1;
	}

}
?>